"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_dom_1 = require("react-dom");
var mobx_react_1 = require("mobx-react");
exports.observer = mobx_react_1.observer;
var utils_1 = require("./utils");
var Cans = (function () {
    function Cans() {
        this.__routerComponent = React.createElement('div');
        this.__plugins = [];
        this.__pluginsObject = {};
        this.__models = [];
        this.__modelsObject = {};
    }
    Cans.prototype.__getInjectList = function () {
        return this.__models.map(function (model) { return model.namespace; }).filter(function (_) { return _; });
    };
    Object.defineProperty(Cans.prototype, "models", {
        get: function () {
            return this.__modelsObject;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Cans.prototype, "plugins", {
        get: function () {
            return this.__pluginsObject;
        },
        enumerable: true,
        configurable: true
    });
    Cans.prototype.route = function (routeFunc) {
        // wrap provider
        var providerProps = {};
        this.__routerComponent = React.createElement(mobx_react_1.Provider, this.models, routeFunc());
    };
    Cans.prototype.model = function (model) {
        // registry model
        var o = typeof model.observable === 'function' ? model.observable(this) : model.observable;
        if (utils_1.isProtected(model)) {
            // every model is protected by default
            utils_1.defineReadOnlyProperty(this.__modelsObject, model.namespace, o, 'This model is readonly');
        }
        else {
            this.__modelsObject[model.namespace] = o;
        }
        this.__models.push(model);
    };
    Cans.prototype.start = function (el) {
        react_dom_1.render(this.__routerComponent, el);
        this.__mountedRoot = el;
    };
    Cans.prototype.use = function (plugin) {
        // registry plugin
        var o = plugin.observable(this);
        if (utils_1.isProtected(plugin)) {
            utils_1.defineReadOnlyProperty(this.__pluginsObject, plugin.namespace, o, 'This plugin is readonly');
        }
        else {
            this.__pluginsObject[plugin.namespace] = plugin.observable(this);
        }
        this.__plugins.push(plugin);
    };
    return Cans;
}());
exports.Cans = Cans;
var createCansApp = function () {
    return new Cans();
};
// model inject helper
exports.inject = function (view) { return mobx_react_1.inject(function (models) { return ({ models: models }); })(mobx_react_1.observer(view)); };
exports.default = createCansApp;
